import rich_click

# rich_click.rich_click.THEME = 'modern'
rich_click.rich_click.MAX_WIDTH = 85

rich_click.rich_click.COMMANDS_PANEL_TITLE = 'commands'
rich_click.rich_click.OPTIONS_PANEL_TITLE = 'options'
rich_click.rich_click.COMMANDS_BEFORE_OPTIONS = True

rich_click.rich_click.STYLE_OPTION = 'bold blue'
rich_click.rich_click.STYLE_ARGUMENT = 'bold blue'
rich_click.rich_click.STYLE_COMMAND = 'bold blue'
rich_click.rich_click.STYLE_SWITCH = 'bold red'
rich_click.rich_click.STYLE_METAVAR = 'bold red'
rich_click.rich_click.STYLE_METAVAR_SEPARATOR = 'dim'
rich_click.rich_click.STYLE_USAGE = 'bold yellow'
rich_click.rich_click.STYLE_USAGE_COMMAND = 'bold'
rich_click.rich_click.STYLE_HELPTEXT_FIRST_LINE = ''
rich_click.rich_click.STYLE_HELPTEXT = 'dim'
rich_click.rich_click.STYLE_OPTION_DEFAULT = 'dim'
rich_click.rich_click.STYLE_REQUIRED_SHORT = 'bold yellow'
rich_click.rich_click.STYLE_REQUIRED_LONG = 'bold yellow'
rich_click.rich_click.STYLE_OPTIONS_PANEL_BORDER = 'dim'
rich_click.rich_click.STYLE_COMMANDS_PANEL_BORDER = 'dim'
